/*
 * C version of aspidefs.inc
 */
#define uchar unsigned char
#define ushort unsigned short
#define ulong unsigned long
/****************************************************************************
 *
 * *** SCSI I/O Request Structure ***
 *
 * Note that all elements in this structure are intended to be stored
 * in Intel byte order.
 *
 * By surmise, this structure is followed by a 6-, 10-, or 12-byte
 * command/data packet (if we are doing an EXEC_SIO).  This buffer is
 * SRCDBLen bytes long.
 *
 * The CD buffer is then followed by SRSnsLen bytes of buffer for sense
 * data.
 *
 * Up to SRBufLen bytes of data is transferred via the SRBufPtr.
 *
 ****************************************************************************/

struct SRBlock {
	uchar SRCmd;		/* Request Command */
	uchar SRStat; 		/* Request Status */
	uchar SRHaId; 		/* Host Adapter ID # */
	uchar SRFlags;		/* Request Flags */
	long pad1;		/* Reserved DWORD */
	uchar SRTarget;		/* Target Id */
	uchar SRLun;		/* Logical Unit Number */
	ulong SRBufLen;		/* Data Allocation Length */
	uchar SRSnsLen;		/* Sense Allocation Length */
	void far *SRBufPtr;	/* Data Buffer Pointer */
	struct SRBlock far *SRNextPtr;	/* Next request pointer for linking */
	uchar SRCDBLen;		/* CDB Length */
	uchar SRHaStat;		/* Host Adapter Status */
	uchar SRTargStat;	/* Target Status */

	void (far *SRPost)();	/* POST routine address */
	ushort SRPostRealDS;	/* Real Mode POST DS */
	void far *SRBPtr;	/* SRB pointer */
	short pad2;		/* Reserved WORD */

	ulong SRPhysAddr;	/* Physical Address of the SRB */

	char SRWkSpace[22];	/* Reserved for SCSI Manager Workspace */
};

struct CDB_SRBBlock {	/* ?? */
	char cdbpad[sizeof(struct SRBlock)];
	char CDBByte[10];
};

/****************************************************************************
 * *** Other command fields ***
 ***************************************************************************/
struct HABlock {	/* HA Inquiry command */
	char habpad[8];
	char HACount;	/* # of Host Adapters */
	uchar HAIdNum;	/* SCSI Id of the host adapter */
	char SCSIMgrId[16];	/* SCSI Manager ID String */
	char HAIdString[16];	/* Host Adapter ID String */
	uchar HAUnique;	/* Host Adapter Unique paramters */
};

struct GDEVBlock {	/* GET DEVICE TYPE COMMAND */
	char gdevpad[10];
	char ScsiDevType;	/* Peripheral Device Type Byte */
};

struct ABORTBlock {	/* ABORT SCSI I/O REQUEST */
	char abtpad[8];
	struct SRBlock far *AbortPtr;	/* Pointer to SRBlock to Abort */
};

struct BIOSBlock {	/* GET BIOS DRIVE INFORMATION */
	char biospad[10];
	uchar BIOSFlags;
	uchar Int13Drive;
	uchar HeadTranslation;
	uchar SectorTranslation;
	uchar BIOSReserved;
};

/****************************************************************************
 * *** Flag Definitions ***
 ****************************************************************************/
#define SRB_POST	0x0001	/* Interrupt POST enable/disable */
#define SRB_LINKED	0x0002	/* Next SRB field valid/invalid */
#define SRB_DIR_IN	0x0008
#define SRB_DIR_OUT	0x0010

/****************************************************************************
 * *** Command Definitions ***
 ****************************************************************************/
#define HA_INQ		0x00
#define GET_TYPE	0x01
#define EXEC_SIO	0x02
#define ABORT_SRB	0x03
#define RESET_DEV	0x04
#define SET_HAPRMS	0x05

/****************************************************************************
 * *** SRStat Status values ***
 ****************************************************************************/
#define SRB_PENDING	0x00
#define SRB_COMP	0x01
#define SRB_ABORTED	0x02
#define SRB_ABORT_FAIL	0x03
#define SRB_ERR 	0x04
#define INV_SPI_CMD	0x80
#define INV_HA_ID	0x81
#define DEV_NOT_INST	0x82

/****************************************************************************
 * *** Host Adapter Status ***
 ****************************************************************************/
#define HASTAT_OK	0x00	/* Host adapter did not detect any error */
#define HASTAT_SELTOUT	0x11	/* Selection Timeout */
#define HASTAT_DO_DU	0x12	/* Data overrun data underrun */
#define HASTAT_UNXBF	0x13	/* Unexpected bus free */
#define HASTAT_BUSFAIL	0x14	/* Target bus phase sequence failure */

/****************************************************************************
 * *** General Purpose Buffer for ASPI I/O
 *
 * Note certain aspects of this: a 64-byte SRBlock is at the top; this
 * gets partially overwritten by ASPI commands other than EXEC_SIO, which
 * uses the command/sense region immediately following.  The location of the
 * sense buffer depends on the size of command packet.
 ***************************************************************************/
typedef union {
	struct SRBlock srb;
	union {
		struct {
			struct SRBlock pad64;
			struct DATA_XFER_6 c6;
			struct SENSE_DATA_FMT s6;
		} cs6;
		struct {
			struct SRBlock pad64;
			struct DATA_XFER_10 c10;
			struct SENSE_DATA_FMT s10;
		} cs10;
		struct {
			struct SRBlock pad64;
			struct DATA_XFER_12 c12;
			struct SENSE_DATA_FMT s12;
		} cs12;
	} cs;
	struct HABlock hab;
	struct GDEVBlock gdev;
	struct ABORTBlock abort;
	struct BIOSBlock bios;
} ASPIbuf;
