/*
 * C version of SCSIDEFS.INC
 *
 ****************************************************************************
 *
 * Name: 	SCSIDEFS.INC
 *
 * Description:	SCSI definitions   (Assembly Language)
 *
 * History  JMS	Version 1.0 4/26/91
 *
 ****************************************************************************/
#define uchar unsigned char

/****************************************************************************
 *		    %%% TARGET STATUS VALUES %%%
 ****************************************************************************/
#define STATUS_GOOD	0x00	/* Status Good */
#define STATUS_CHKCOND	0x02	/* Check Condition */
#define STATUS_CONDMET	0x04	/* Condition Met */
#define STATUS_BUSY	0x08	/* Busy */
#define STATUS_INTERM	0x10	/* Intermediate */
#define STATUS_INTCDMET 0x14	/* Intermediate-condition met */
#define STATUS_RESCONF	0x18	/* Reservation conflict */
#define STATUS_COMTERM	0x22	/* Command Terminated */
#define STATUS_QFULL	0x28	/* Queue full */


/****************************************************************************
 *		 %%%  SCSI MISCELLANIOUS EQUATES   %%%
 ****************************************************************************/
#define MAXLUN		7	/* Maximum Logical Unit Id */
#define MAXTARG 	7	/* Maximum Target Id */
#define MAX_SCSI_LUNS	64	/* Maximum Number of SCSI LUNs */
#define MAX_NUM_HA	8	/* Maximum Number of SCSI HA's */

#define LF		10	/* Linefeed */
#define CR		13	/* Carriage Return */
/***************************************************************************
 *
 *		     %%% SCSI COMMAND OPCODES %%%
 *
 **************************************************************************/

/****************************************************************************
 *		   %%% Commands for all Device Types %%%
 ****************************************************************************/
#define SCSI_Change_Def 0x040	/* Change Definition (Optional) */
#define SCSI_Compare	0x039	/* Compare (O) */
#define SCSI_Copy	0x018	/* Copy (O) */
#define SCSI_Cop_Verify 0x03A	/* Copy and Verify (O) */
#define SCSI_Inquiry	0x012	/* Inquiry (MANDATORY) */
#define SCSI_Log_Select 0x04C	/* Log Select (O) */
#define SCSI_Log_Sense	0x04D	/* Log Sense (O) */
#define SCSI_Mod_Sel_6	0x015	/* Mode Select 6-byte (Device Specific) */
#define SCSI_Mod_Sel_10 0x055	/* Mode Select 10-byte (Device Specific) */
#define SCSI_Mod_Sen_6	0x015	/* Mode Sense 6-byte (Device Specific) */
#define SCSI_Mod_Sen_10 0x015	/* Mode Sense 10-byte (Device Specific) */
#define SCSI_Read_Buff	0x03C	/* Read Buffer (O) */
#define SCSI_Req_Sense	0x003	/* Request Sense (MANDATORY) */
#define SCSI_Send_Diag	0x01D	/* Send Diagnostic (O) */
#define SCSI_Tst_U_Rdy	0x000	/* Test Unit Ready (MANDATORY) */
#define SCSI_Write_Buff 0x03B	/* Write Buffer (O) */

/****************************************************************************
 *	       %%% Commands Unique to Direct Access Devices %%%
 ****************************************************************************/
#define SCSI_Compare	0x039	/* Compare (O) */
#define SCSI_Format	0x004	/* Format Unit (MANDATORY) */
#define SCSI_Lck_Un_Cac 0x036	/* Lock Unlock Cache (O) */
#define SCSI_Prefetch	0x034	/* Prefetch (O) */
#define SCSI_Med_Removl 0x01E	/* Prevent/Allow medium Removal (O) */
#define SCSI_Read_6	0x008	/* Read 6-byte (MANDATORY) */
#define SCSI_Read_10	0x028	/* Read 10-byte (MANDATORY) */
#define SCSI_Rd_Capac	0x025	/* Read Capacity (MANDATORY) */
#define SCSI_Rd_Defect	0x037	/* Read Defect Data (O) */
#define SCSI_Read_Long	0x03E	/* Read Long (O) */
#define SCSI_Reass_Blk	0x007	/* Reassign Blocks (O) */
#define SCSI_Rcv_Diag	0x01C	/* Receive Diagnostic Results (O) */
#define SCSI_Release	0x017	/* Release Unit (MANDATORY) */
#define SCSI_Rezero	0x001	/* Rezero Unit (O) */
#define SCSI_Srch_Dat_E 0x01D	/* Search Data Equal (O) */
#define SCSI_Srch_Dat_H 0x01D	/* Search Data High (O) */
#define SCSI_Srch_Dat_L 0x01D	/* Search Data Low (O) */
#define SCSI_Seek_6	0x00B	/* Seek 6-Byte (O) */
#define SCSI_Seek_10	0x02B	/* Seek 10-Byte (O) */
#define SCSI_Send_Diag	0x01D	/* Send Diagnostics (MANDATORY) */
#define SCSI_Set_Limit	0x033	/* Set Limits (O) */
#define SCSI_Start_Stp	0x01B	/* Start/Stop Unit (O) */
#define SCSI_Sync_Cache 0x035	/* Synchronize Cache (O) */
#define SCSI_Verify	0x02F	/* Verify (O) */
#define SCSI_Write_6	0x00A	/* Write 6-Byte (MANDATORY) */
#define SCSI_Write_10	0x02A	/* Write 10-Byte (MANDATORY) */
#define SCSI_Wrt_Verify 0x02E	/* Write and Verify (O) */
#define SCSI_Write_Long 0x03F	/* Write Long (O) */
#define SCSI_Write_Same 0x041	/* Write Same (O) */

/****************************************************************************
 *	      %%% Commands Unique to Sequential Access Devices %%%
 ****************************************************************************
#define SCSI_Erase	0x019	/* Erase (MANDATORY) */
#define SCSI_Load_Un	0x01B	/* Load/Unload (O) */
#define SCSI_Locate	0x02B	/* Locate (O) */
#define SCSI_Rd_Blk_Lim 0x005	/* Read Block Limits (MANDATORY) */
#define SCSI_Read_Pos	0x034	/* Read Position (O) */
#define SCSI_Read_Rev	0x00F	/* Read Reverse (O) */
#define SCSI_Rec_Bf_Dat 0x014	/* Recover Buffer Data (O) */
#define SCSI_Reserve	0x016	/* Reserve Unit (MANDATORY) */
#define SCSI_Rewind	0x001	/* Rewind (MANDATORY) */
#define SCSI_Space	0x011	/* Space (MANDATORY) */
#define SCSI_Verify_T	0x013	/* Verify (Tape) (O) */
#define SCSI_Wrt_File	0x010	/* Write Filemarks (MANDATORY) */

/**************************************************************************** 
 *		  %%% Commands Unique to Printer Devices %%%
 ****************************************************************************/
#define SCSI_Print	0x00A	/* Print (MANDATORY) */
#define SCSI_Slew_Pnt	0x00B	/* Slew and Print (O) */
#define SCSI_Stop_Pnt	0x01B	/* Stop Print (O) */
#define SCSI_Sync_Buff	0x010	/* Synchronize Buffer (O) */

/****************************************************************************
 *		 %%% Commands Unique to Processor Devices %%%
 ****************************************************************************
#define SCSI_Receive	0x008	/* Receive (O) */
#define SCSI_Send	0x00A	/* Send (O) */

/****************************************************************************
 *		 %%% Commands Unique to Write-Once Devices %%%
 ****************************************************************************
#define SCSI_Medium_Scn 0x038	/* Medium Scan (O) */
#define SCSI_SrchDatE10 0x031	/* Search Data Equal 10-Byte (O) */
#define SCSI_SrchDatE12 0x0B1	/* Search Data Equal 12-Byte (O) */
#define SCSI_SrchDatH10 0x030	/* Search Data High 10-Byte (O) */
#define SCSI_SrchDatH12 0x0B0	/* Search Data High 12-Byte (O) */
#define SCSI_SrchDatL10 0x032	/* Search Data Low 10-Byte (O) */
#define SCSI_SrchDatL12 0x0B2	/* Search Data Low 12-Byte (O) */
#define SCSI_Set_Lim_10 0x033	/* Set Limits 10-Byte (O) */
#define SCSI_Set_Lim_12 0x0B3	/* Set Limits 10-Byte (O) */
#define SCSI_Verify_10	0x02F	/* Verify 10-Byte (O) */
#define SCSI_Verify_12	0x0AF	/* Verify 12-Byte (O) */
#define SCSI_Write_12	0x0AA	/* Write 12-Byte (O) */
#define SCSI_Wrt_Ver_10 0x02E	/* Write and Verify 10-Byte (O) */
#define SCSI_Wrt_Ver_12 0x0AE	/* Write and Verify 12-Byte (O) */

/****************************************************************************
 *		  %%% Commands Unique to CD-ROM Devices %%%
 ****************************************************************************/

#define SCSI_PlayAud_10 0x045	/* Play Audio 10-Byte (O) */
#define SCSI_PlayAud_12 0x0A5	/* Play Audio 12-Byte 12-Byte (O) */
#define SCSI_PlayAudMsf 0x047	/* Play Audio MSF (O) */
#define SCSI_PlayA_TkIn 0x048	/* Play Audio Track/Index (O) */
#define SCSI_PlyTkRel10 0x049	/* Play Track Relative 10-Byte (O) */
#define SCSI_PlyTkRel12 0x0A9	/* Play Track Relative 12-Byte (O) */
#define SCSI_ReadCDCap	0x025	/* Read CD-ROM Capacity (MANDATORY) */
#define SCSI_ReadHeader 0x044	/* Read Header (O) */
#define SCSI_SubChannel 0x042	/* Read Subchannel (O) */
#define SCSI_Read_TOC	0x043	/* Read TOC (O) */

/****************************************************************************
 *		  %%% Commands Unique to Scanner Devices %%%
 ****************************************************************************/

#define SCSI_GetDBStat	0x034	/* Get Data Buffer Status (O) */
#define SCSI_GetWindow	0x025	/* Get Window (O) */
#define SCSI_ObjectPos	0x031	/* Object Postion (O) */
#define SCSI_Scan	0x01B	/* Scan (O) */
#define SCSI_SetWindow	0x024	/* Set Window (MANDATORY) */

/****************************************************************************
 *	       %%% Commands Unique to Optical Memory Devices %%%
 ****************************************************************************/

#define SCSI_UpdateBlk	0x3D	/* Update Block (O) */

/****************************************************************************
 *	       %%% Commands Unique to Medium Changer Devices %%%
 ****************************************************************************/

#define SCSI_ExchMedium 0x0A6	/* Exchange Medium (O) */
#define SCSI_InitElStat 0x007	/* Initialize Element Status (O) */
#define SCSI_PosToElem	0x02B	/* Position to Element (O) */
#define SCSI_Req_VE_Add 0x0B5	/* Request Volume Element Address (O) */
#define SCSI_SendVolTag 0x0B6	/* Send Volume Tag (O) */

/****************************************************************************
 *	      %%% Commands Unique to Communication Devices %%%
 ****************************************************************************/

#define SCSI_Get_Msg_6	0x008	/* Get Message 6-Byte (MANDATORY) */
#define SCSI_Get_Msg_10 0x028	/* Get Message 10-Byte (O) */
#define SCSI_Get_Msg_12 0x0A8	/* Get Message 12-Byte (O) */
#define SCSI_Snd_Msg_6	0x00A	/* Send Message 6-Byte (MANDATORY) */
#define SCSI_Snd_Msg_10 0x02A	/* Send Message 10-Byte (O) */
#define SCSI_Snd_Msg_12 0x0AA	/* Send Message 12-Byte (O) */

/****************************************************************************
 *
 *		  %%%  End Of SCSI Command Opcodes  %%%
 *
 ***************************************************************************/

/****************************************************************************
 *		%%%    Request Sense Data Format   %%%
 * N.B. -- there seems to have been some problems in the .INC version, so
 * I have truncated this to a 'standard' extended sense.  This may have to
 * do with differences between SCSI-1 & SCSI-2.
 ****************************************************************************/
struct SENSE_DATA_FMT {
	uchar ErrorCode;	/* Error Code (70H or 71H) */
	uchar SegmentNum;	/* Number of current segment descriptor */
	uchar SenseKey;		/* Sense Key(See bit definitions too) */
	uchar Info3;		/* Information MSB byte 3 */
	uchar Info2;		/* Information MSB byte 2 */
	uchar Info1;		/* Information MSB byte 1 */
	uchar Info0;		/* Information MSB byte 0 */
	uchar AddSenLen;	/* Additional Sense Length */
	uchar ComSpecInf[4];	/* Command Specific Information bytes */
	uchar AddSenseCode;	/* Additional Sense Code */
	uchar AddSenQual;	/* Additional Sense Code Qualifier */
	/* other sense data can go here...we reserve a total of 64 bytes */
	/*char MoreSense[50];*/
};

/******************************************************************************
 *			REQUEST SENSE ERROR CODE
 *****************************************************************************/
#define SERROR_CURRENT	0x070	/* Current Errors */
#define SERROR_DEFERED	0x071	/* Deferred Errors */

/******************************************************************************
 *			REQUEST SENSE BIT DEFINITIONS
 *****************************************************************************/
#define SENSE_VALID	0x080	/* Byte 0 Bit 7 */
#define SENSE_FILEMRK	0x080	/* Byte 2 Bit 7 */
#define SENSE_EOM	0x040	/* Byte 2 Bit 6 */
#define SENSE_ILI	0x020	/* Byte 2 Bit 5 */
#define SENSE_KEY	0x00F	/* Byte 2 Bits 0-3 */

/*****************************************************************************
 *			REQUEST SENSE SENSE KEY DEFINITIONS
 ****************************************************************************/

#define KEY_NOSENSE	0x00	/* No Sense */
#define KEY_RECERROR	0x01	/* Recovered Error */
#define KEY_NOTREADY	0x02	/* Not Ready */
#define KEY_MEDIUMERR	0x03	/* Medium Error */
#define KEY_HARDERROR	0x04	/* Hardware Error */
#define KEY_ILLGLREQ	0x05	/* Illegal Request */
#define KEY_UNITATT	0x06	/* Unit Attention */
#define KEY_DATAPROT	0x07	/* Data Protect */
#define KEY_BLANKCHK	0x08	/* Blank Check */
#define KEY_VENDSPEC	0x09	/* Vendor Specific */
#define KEY_COPYABORT	0x0A	/* Copy Abort */
#define KEY_EQUAL	0x0C	/* Equal (Search) */
#define KEY_VOLOVRFLW	0x0D	/* Volume Overflow */
#define KEY_MISCOMP	0x0E	/* Miscompare (Search) */
#define KEY_RESERVED	0x0F	/* Reserved */

/****************************************************************************
 *		     %%%    Inquiry Data Format   %%%
 ****************************************************************************/
struct INQRY_DATA_FMT {
	uchar PerQualTyp;	/* Peripheral Qualifier and Device Type */
	uchar Inq_Flags;	/* Reserved except for bit 7 (RMB) */
	uchar Versions;		/* ISO, ECMA and ANSI versions */
	char Reserved0;
	uchar AddLength;	/* Additional Length */
	uchar Reserved1;	/* Reserved */
	uchar Reserved2;	/* Reserved */
	uchar Flags;	/* RelAdr, WBus32, WBus16, Sync, .... */
	uchar VendorId;	/* Command Specific Information LSB */
	uchar ProdId;	/* Additional Sense Code */
	uchar ProdRev;	/* Additional Sense Code Qualifier */
};

/****************************************************************************
 *		  %%% PERIPHERAL DEVICE TYPE DEFINITIONS %%%
 ****************************************************************************
#define DTYPE_DASD	0x00	/* Disk Device */
#define DTYPE_SEQD	0x01	/* Tape Device */
#define DTYPE_PRNT	0x02	/* Printer */
#define DTYPE_PROC	0x03	/* Processor */
#define DTYPE_WORM	0x04	/* Write-once read-multiple */
#define DTYPE_CROM	0x05	/* CD-ROM device */
#define DTYPE_SCAN	0x06	/* Scanner device */
#define DTYPE_OPTI	0x07	/* Optical memory device */
#define DTYPE_JUKE	0x08	/* Medium Changer device */
#define DTYPE_COMM	0x09	/* Communications device */
#define DTYPE_RESL	0x0A	/* Reserved (low) */
#define DTYPE_RESH	0x1E	/* Reserved (high) */
#define DTYPE_NONE	0x1F	/* Unknown or no device type */

/****************************************************************************
 *		  %%% ANSI APPROVED VERSION DEFINITIONS %%%
 ****************************************************************************/

#define ANSI_MAYBE	0x00	/* May or may not be ANSI approved stand. */
#define ANSI_SCSI1	0x01	/* Complies to ANSI X3.131-1986 (SCSI-1) */
#define ANSI_SCSI2	0x02	/* Device complies to SCSI-2 */
#define ANSI_RESLO	0x03	/* Reserved (low) */
#define ANSI_RESHI	0x07	/* Reserved (high) */

/*
 * Transfer command packets: these are altered from the assembler form,
 * which defined the 6-byte structure and implemented the others as offsets
 * from its _OpCode field.  Also, since the SRB is part of ASPI, not of
 * SCSI per se, we make no reference to it here.
 */
struct DATA_XFER_6 {
	uchar OpCode;	/* Opcode of SCSI Command */
	uchar LUNumber;	/* Logical Unit Number & LBA MSB */
	uchar wLBA1;	/* word Logical Block Address byte 1 */
	uchar wLBA0;	/* word Logical Block Address byte 0 */
	uchar bLength;	/* byte Length */
	uchar Control6;	/* Control byte for 6-byte packets */
};
/* defs for tapes, which use a 24-bit length here */
#define tLength2 wLBA1
#define tLength1 wLBA0
#define tLength0 bLength

struct DATA_XFER_10 {
	uchar OpCode;	/* Opcode of SCSI Command */
	uchar LUNumber;	/* Logical Unit Number & LBA MSB */
	uchar XFpad;	/* reserved */
	uchar lLBA3;	/* long Logical Block Address byte 3 */
	uchar lLBA2;	/* long Logical Block Address byte 2 */
	uchar lLBA1;	/* long Logical Block Address byte 1 */
	uchar lLBA0;	/* long Logical Block Address byte 0 */
	uchar wLength1;	/* word Length byte 1 */
	uchar wLength0;	/* word Length byte 0 */
	uchar Control10;	/* Control byte for 10-byte packets */
};
struct DATA_XFER_12 {
	uchar OpCode;	/* Opcode of SCSI Command */
	uchar LUNumber;	/* Logical Unit Number & LBA MSB */
	uchar XFpad;	/* reserved */
	uchar lLBA3;	/* long Logical Block Address byte 3 */
	uchar lLBA2;	/* long Logical Block Address byte 2 */
	uchar lLBA1;	/* long Logical Block Address byte 1 */
	uchar lLBA0;	/* long Logical Block Address byte 0 */
	uchar lLength3;	/* long Length byte 3 */
	uchar lLength2;	/* long Length byte 2 */
	uchar lLength1;	/* long Length byte 1 */
	uchar lLength0;	/* long Length byte 0 */
	uchar Control12;	/* Control byte for 12-byte packets */
};
